#!/usr/bin/bash

DATE=`date '+%Y%m%d_%H%M%S'`

this=benchmark_wguide_cpu
log=$this.log
savedir=$DATE

echo "### $this, $DATE, $log ###"
echo "arg: method"
echo "arg: $1"

mth=$1
sys=wguide

bin_list=( ../pm0 \
	   ../pm1 \
	   ../pm2 \
	   ../pm3 )

src_list=( ./${mth} )

cfg_list=( ${sys}_${mth}_3D_8x8x8.cfg \
           ${sys}_${mth}_3D_16x16x16.cfg \
           ${sys}_${mth}_3D_32x32x32.cfg \
	   ${sys}_${mth}_3D_64x64x64.cfg \
	   ${sys}_${mth}_3D_128x128x128.cfg \
	   ${sys}_${mth}_3D_256x256x256.cfg )

# options
compare="-c"
store="-r ${this}.csv"
cpu="-p"
gpu="-g"
quiet="-q"
iterations="-n 3 --skipFirst"
store_field="-f"
profOnly="--profOnly"

profile=3

for src in ${src_list[@]}; do
    out_dir="--outDir $src"
    for bin in ${bin_list[@]}; do
	if ! test -f "$bin"
	then
	    echo "$bin does not exist." | tee -a ${mth}/${log};
	    exit;
	fi
	for cfg in ${cfg_list[@]}; do
	    if ! test -f "$src/$cfg"
	    then
		echo "$src/$cfg does not exist." | tee -a ${mth}/${log};
		exit;
	    fi

	    if [[ $profile -eq 1 ]]
	    then
		echo "$bin $quiet -i $src/$cfg $gpu $compare $out_dir $store 2>&1" | tee -a ${mth}/${log};
		$bin $quiet -i $src/$cfg $gpu $compare $out_dir $store  2>&1 | tee -a ${mth}/${log};
	    fi
	    
	    if [[ $profile -eq 2 ]]
	    then
		echo "$bin $quiet -i $src/$cfg $gpu $iterations $compare $out_dir $store 2>&1" | tee -a ${mth}/${log};
		$bin $quiet -i $src/$cfg $gpu $iterations $compare $out_dir $store  2>&1 | tee -a ${mth}/${log};
	    fi

	    if [[ $profile -eq 3 ]]
	    then
	       echo "$bin $quiet -i $src/$cfg $gpu $iterations $profOnly $out_dir $store 2>&1" | tee -a ${mth}/${log};
	       $bin $quiet -i $src/$cfg $gpu $iterations $profOnly $out_dir $store  2>&1 | tee -a ${mth}/${log};
	    fi
	    
	    if [[ $profile -eq 4 ]]
	    then
	       echo "$bin $quiet -i $src/$cfg $gpu $profOnly $out_dir $store 2>&1" | tee -a ${mth}/${log};
	       $bin $quiet -i $src/$cfg $gpu $profOnly $out_dir $store  2>&1 | tee -a ${mth}/${log};
	    fi	    
	    
	done
    done
done
echo "finished." | tee -a $log; 
