#!/usr/bin/bash

DATE=`date '+%Y.%m.%d'`
TIME=`date '+%H:%M:%S'`

mth=vwpm

this=flux_benchmark_cpu
log=$this.log

bin_list=( ../pm )

cfg_extensions=( n1p01 \
		 n1p1 \
#		 n1p2 \
#		 n1p3 \
#		 n1p4 \
		 n1p5 \
#		 n1p6 \
#		 n1p7 \
#		 n1p8 \
#		 n1p9 \
		 n2p0 \
#		 n2p1 \
#		 n2p2 \
#		 n2p3 \
#		 n2p4 \
		 n2p5 \
	       )

save_files=( flux_report.csv \
  	     N_preRun.csv \
  	     N_postRun.csv \
	     spectrum_report.csv \
	     spectrum_summary.csv \
	     mode_report.csv \
	     mode_summary.csv \
	     divd_report.csv \
	     divd_summary.csv \
	     PROF_FLUX_cpu.csv \
	     PROF_EX_cpu.csv \
	     PROF_EY_cpu.csv \
	     PROF_EZ_cpu.csv \
	   )

system_list=( homog \
	      boundary \
	      wguide_rect_planeWave \
	      wguide_rect_gaussBeam \
            )

# ea - evanescent modes like propagating modes (i.e. apply complex calculus without taking care of evanescent wave model)
# t9 - Evanescent Boundary Threshold (set to 89.0° <= theta <= 91.1° is assigned 90° to avoid instabilities in the z-component)
# ev - Geometric Evanescent Wave Model (Josefowski 2007)
# ny - Anti-Aliasing Nyquist Filter
# se - Skip Evanescent Modes by manual override (see flux.cfg)
# dd - Divergence of D (First Maxwell Equation)
# sp - Spatial Filter (Mean-Average or Gaussian) with Sigma if Gaussian and Size filter_nx x filter_ny (see flux.cfg)
#      0 = Mean Average Filter
#      2 = Gaussian Filter
# lp - Low Pass Filter w/ bandlimit at +/- N/2 modes
# al - Adaptive Low Pass Filter

flux_config_prefix=flux_config
                                                      # ea t9 ev ny se dd sp lp al Comment  
flux_config_list=( ${flux_config_prefix}_1_1 \       # x                     
		   ${flux_config_prefix}_1_2 \       # x  x
		   ${flux_config_prefix}_2_1 \       #       x
		   ${flux_config_prefix}_2_2 \       #    x  x
		   ${flux_config_prefix}_3_1 \       #    x  x  x 
		   ${flux_config_prefix}_3_2 \       #    x  x  x  x
		   ${flux_config_prefix}_4_1 \       #    x  x        x
		   ${flux_config_prefix}_4_2 \       #    x  x  x     x
		   ${flux_config_prefix}_4_3 \       #    x  x  x  x  x
		   ${flux_config_prefix}_5_1_1 \     #    x  x           0        7x7 
		   ${flux_config_prefix}_5_1_2 \     #    x  x        x  0        7x7 
		   ${flux_config_prefix}_5_1_3 \     #    x  x  x     x  0        7x7
		   ${flux_config_prefix}_5_1_4 \     #    x  x  x     x  0        7x7 
		   ${flux_config_prefix}_5_2_1 \     #    x  x           2        7x7 Sigma=1 
		   ${flux_config_prefix}_5_2_2 \     #    x  x        x  2        7x7 Sigma=1
		   ${flux_config_prefix}_5_2_3 \     #    x  x  x        2        7x7 Sigma=1
		   ${flux_config_prefix}_5_2_4 \     #    x  x  x     x  2        7x7 Sigma=1
		   ${flux_config_prefix}_6_1 \       #    x  x           0  x     7x7, N=20
		   ${flux_config_prefix}_6_2 \       #    x  x           0  x     7x7, N=12 
		   ${flux_config_prefix}_6_3 \       #    x  x           0  x     7x7, N=6
		   #${flux_config_prefix}_7 \         # Adaptive Filter (dynamic mode limitation) (TODO)
		 )

# pm options
compare="-c"
store="-r ${this}.csv"
cpu="-p"
gpu="-g"
quiet="-q"
iterations="-n 1 --skipFirst"
field="-f -x 1 -l 48"

for bin in ${bin_list[@]}; do
    
    if ! test -f "$bin"
    then
	echo "$bin does not exist."
	exit;
    fi

    for flux_config in ${flux_config_list[@]}; do

	flux_config_dir=./${mth}/${flux_config}
	
	if ! test -d "$flux_config_dir"
	then
	    echo "$flux_config_dir does not exist."
	    exit;
	fi
	
	for system in ${system_list[@]}; do
	    
	    cfg_dir=./${mth}/${system}       
	    cfg_basename=${system}_${mth}_3D_96x96x96	
	    
	    for ext in ${cfg_extensions[@]}; do

		cfg=${cfg_basename}_${ext}.cfg
		
		save_dir=${mth}/${flux_config}/${system}/$ext
		if ! test -d "$save_dir"
		then
		    echo "Creating $save_dir ... "
		    mkdir -p $save_dir;
		fi

		echo "done."

		log_dir=$save_dir	    
		echo "$this, $filter_type, $DATE, $TIME, $log_dir/$log" | tee $log_dir/$log;
		
		if ! test -f "$cfg_dir/$cfg"
		then
		    echo "$cfg_dir/$cfg does not exist." | tee -a $log_dir/$log;
		    exit;
		fi

		# === cleanup ===
		
		echo "Cleanup duplicate report files from last run and clean save dir ..." | tee -a $log_dir/$log;
		
		echo "rm PROF_*_cpu.csv 2>&1 | tee -a $log_dir/$log;"  | tee -a $log_dir/$log; 
		rm PROF_*_cpu.csv 2>&1 | tee -a $log_dir/$log;
				
		for file in ${save_files[@]}; do

		    if test -f "$file"
		    then
			
			echo "rm $file 2>&1 | tee -a $log_dir/$log" | tee -a $log_dir/$log;
			rm $file 2>&1 | tee -a $log_dir/$log;

		    fi

		done

		echo "done." | tee -a $log_dir/$log;

		# === prepare cfg-file ===
		
		echo "Prepare config ..." | tee -a $log_dir/$log;

		echo "cp ${cfg_dir}/$cfg ./${save_dir} 2>&1 | tee -a $log_dir/$log;" | tee -a $log_dir/$log;
		cp ${cfg_dir}/$cfg ./${save_dir} 2>&1 | tee -a $log_dir/$log;

		echo "cat ${mth}/${flux_config}/flux.cfg >> ${save_dir}/$cfg 2>&1 | tee -a $log_dir/$log;" | tee -a $log_dir/$log;
		cat ${mth}/${flux_config}/flux.cfg >> ./${save_dir}/$cfg 2>&1 | tee -a $log_dir/$log;	     
		
		echo "done." | tee -a $log_dir/$log;

		# === create rerun script ===

		echo "Create rerun script ..." | tee -a $log_dir/$log;
		echo "mkdir ${save_dir}/rerun | tee -a $log_dir/$log;" | tee -a $log_dir/$log;
		mkdir ${save_dir}/rerun 2>&1 | tee -a $log_dir/$log; 
		echo "../pm -i ../$cfg $cpu $compare $field --outDir . $store | tee -a ./rerun.log" | tee -a $log_dir/$log;

		echo "#!/usr/bin/bash" | tee ${save_dir}/rerun/rerun.sh
		echo "# Adapt to your bash bin-directory if needed. Try 'which bash' in a termal to get the correct path." | tee -a ${save_dir}/rerun/rerun.sh
		echo "../pm -i ../$cfg $cpu $compare $field --outDir . $store | tee -a ./rerun.log" | tee -a ${save_dir}/rerun/rerun.sh

		echo "... done." | tee -a $log_dir/$log;
		
		# === run simulation ===

		echo "Run simulation ..." | tee -a $log_dir/$log;

		outDir="--outDir ${mth}/${flux_config}"
		echo "$bin -i ${save_dir}/$cfg $cpu $compare $field $outDir $store 2>&1 | tee -a $log_dir/$log" | tee -a $log_dir/$log;	   	
		$bin -i ${save_dir}/$cfg $cpu $compare $field $outDir $store 2>&1 | tee -a $log_dir/$log;

		echo "done."  | tee -a $log_dir/$log;

	       	# === save executable for rerun ===

		echo "Save executable for rerun ..." | tee -a $log_dir/$log;
		echo "cp -p $bin ${mth}/${flux_config} 2>&1 | tee -a $log_dir/$log;" | tee -a $log_dir/$log;
		cp -p $bin ${mth}/${flux_config} 2>&1 | tee -a $log_dir/$log;		
		
		# === save results ===

		echo "Save results ..." | tee -a ${log_dir}/$log;
		
		for file in ${save_files[@]}; do
		    
		    if test -f "$file"
		    then

			echo "cp -p $file $save_dir | tee -a $log_dir/$log" | tee -a $log_dir/$log;
			cp -p $file $save_dir | tee -a $log_dir/$log;

			echo "rename -f 's/PROF_//' $save_dir/$file | tee -a $log_dir/$log" | tee -a $log_dir/$log;
			rename -f 's/PROF_//' $save_dir/$file | tee -a $log_dir/$log;

		    fi
		    
		done

		echo "done." | tee -a $log_dir/$log;

	    done	
	    
	done
	
	echo "Save flux summary ..." | tee -a ${mth}/${flux_config}/$log;
	echo "cp PROF_flux_per_layer_cpu.csv ${mth}/${flux_config}/flux_summary.csv" | tee -a ${mth}/${flux_config}/$log;
	cp PROF_flux_per_layer_cpu.csv ${mth}/${flux_config}/flux_summary.csv | tee -a ${mth}/${flux_config}/$log;
	
    done
    
done
rm ./*.csv
echo "finished."

