#include <cufft.h>
#include <cuda_runtime.h>
#include <cuda.h>
#include <math.h>

#ifndef TOLERANCE
#define TOLERANCE 1E-4
#endif


/* kernels and wrappers */

/*
 *  3D vectorial
 */

__global__ void YOUR_kernel_3D( cufftDoubleComplex* SX, cufftDoubleComplex* SY, cufftDoubleComplex* SZ,
			        cufftDoubleComplex* EX, cufftDoubleComplex* EY, cufftDoubleComplex* EZ, double* F, 
                                cufftDoubleComplex* N, 
			        cufftDoubleComplex* KX, double* X, 
			        cufftDoubleComplex* KY, double* Y, 
			        double k0, double dz, double dx, double dy, int nx, int ny, int k, 
                                bool do_evanescent_modes = false);
__global__ void YOUR_kernel_3D_2( cufftDoubleComplex* SX, cufftDoubleComplex* SY, cufftDoubleComplex* SZ,
				  cufftDoubleComplex* EX, cufftDoubleComplex* EY, cufftDoubleComplex* EZ, double* F, 
				  cufftDoubleComplex* N, 
				  cufftDoubleComplex* KX, double* X, 
				  cufftDoubleComplex* KY, double* Y, 
				  double k0, double dz, double dx, double dy, int nx, int ny, int k, 
				  bool do_evanescent_modes = false);
void YOUR_kernelWrapper_3D( int blockSize,
                            cufftDoubleComplex* EX, cufftDoubleComplex* EY, cufftDoubleComplex* EZ, double* F, 
                            const cufftDoubleComplex* host_N, cufftDoubleComplex* device_N, 
			    cufftDoubleComplex* KX, double* X, 
			    cufftDoubleComplex* KY, double* Y, 
                            double k0, double dz, double dy, double dx, int nz, int ny, int nx, 
                            bool do_evanescent_modes = false,
			    bool is_debug = false);

/*
 *  3D scalar
 */

__global__ void YOUR_kernel_3D( cufftDoubleComplex* S, 
	   		        cufftDoubleComplex* E, double* F, 
                                cufftDoubleComplex* N, 
			        cufftDoubleComplex* KX, double* X, 
			        cufftDoubleComplex* KY, double* Y, 
                                double k0, double dz, double dx, double dy, int nx, int ny, int k, 
                                bool do_evanescent_modes = false, bool is_TE = true );
__global__ void YOUR_kernel_3D_2( cufftDoubleComplex* S, 
				  cufftDoubleComplex* E, double* F, 
				  cufftDoubleComplex* N, 
				  cufftDoubleComplex* KX, double* X, 
				  cufftDoubleComplex* KY, double* Y, 
				  double k0, double dz, double dx, double dy, int nx, int ny, int k, 
				  bool do_evanescent_modes = false, bool is_TE = true );
void YOUR_kernelWrapper_3D( int blockSize,
                            cufftDoubleComplex* E, double* F, 
                            const cufftDoubleComplex* host_N, cufftDoubleComplex* device_N, 
			    cufftDoubleComplex* KX, double* X, 
			    cufftDoubleComplex* KY, double* Y, 
                            double k0, double dz, double dy, double dx, int nz, int ny, int nx, 
                            bool do_evanescent_modes = false, bool is_TE = true ,
			    bool is_debug = false);

/*
 *  2D vectorial
 */

__global__ void YOUR_kernel_2D( cufftDoubleComplex* SX, cufftDoubleComplex* SY, cufftDoubleComplex* SZ, 
	   	                cufftDoubleComplex* EX, cufftDoubleComplex* EY, cufftDoubleComplex* EZ, double* F, 
				cufftDoubleComplex* N, 
				cufftDoubleComplex* KX, double* X, 
				double k0, double dz, double dx, int nx, int k, 
				bool do_evanescent_modes = false );
__global__ void YOUR_kernel_2D_2( cufftDoubleComplex* SX, cufftDoubleComplex* SY, cufftDoubleComplex* SZ, 
				  cufftDoubleComplex* EX, cufftDoubleComplex* EY, cufftDoubleComplex* EZ, double* F, 
				  cufftDoubleComplex* N, 
				  cufftDoubleComplex* KX, double* X, 
				  double k0, double dz, double dx, int nx, int k, 
				  bool do_evanescent_modes = false );
void YOUR_kernelWrapper_2D( int blockSize,
                            cufftDoubleComplex* EX, cufftDoubleComplex* EY, cufftDoubleComplex* EZ, double* F, 
			    const cufftDoubleComplex* host_N, cufftDoubleComplex* device_N, 
			    cufftDoubleComplex* KX, double* X, 
			    double k0, double dz, double dx, int nz, int nx, 
			    bool do_evanescent_modes = false,
			    bool is_debug = false);

/*
 *  2D scalar
 */

__global__ void YOUR_kernel_2D( cufftDoubleComplex* S, 
	   			cufftDoubleComplex* E, double* F, 
				cufftDoubleComplex* N, 
	   			cufftDoubleComplex* KX, double* X, 
				double k0, double dz, double dx, int nx, int k, 
				bool do_evanescent_modes = false, bool is_TE = true );
__global__ void YOUR_kernel_2D_2( cufftDoubleComplex* S, 
				  cufftDoubleComplex* E, double* F, 
				  cufftDoubleComplex* N, 
				  cufftDoubleComplex* KX, double* X, 
				  double k0, double dz, double dx, int nx, int k, 
				  bool do_evanescent_modes = false, bool is_TE = true );
void YOUR_kernelWrapper_2D( int blockSize,
			    cufftDoubleComplex* E, double* F, 
     			    const cufftDoubleComplex* host_N, cufftDoubleComplex* device_N, 
			    cufftDoubleComplex* KX, double* X, 
			    double k0, double dz, double dx, int nz, int nx, 
			    bool do_evanescent_modes = false, bool is_TE = true,
			    bool is_debug = false);


/* complex math functions */
__host__ __device__ static __inline__ cuDoubleComplex cuCmul(cuDoubleComplex x, double y);
__host__ __device__ static __inline__ cuDoubleComplex cuCdiv(cuDoubleComplex x, double y);
__host__ __device__ static __inline__ cuDoubleComplex cuCexp(cuDoubleComplex x);
__host__ __device__ static __inline__ cuDoubleComplex cuCsqrt(cuDoubleComplex x);

/* overload */
__host__ __device__ static __inline__ cuDoubleComplex operator*(cuDoubleComplex x, double y);
__host__ __device__ static __inline__ cuDoubleComplex operator/(cuDoubleComplex x, double y);
__host__ __device__ static __inline__ cuDoubleComplex operator+(cuDoubleComplex x, cuDoubleComplex y);
__host__ __device__ static __inline__ cuDoubleComplex operator-(cuDoubleComplex x, cuDoubleComplex y);
__host__ __device__ static __inline__ cuDoubleComplex operator*(cuDoubleComplex x, cuDoubleComplex y);
__host__ __device__ static __inline__ cuDoubleComplex operator/(cuDoubleComplex x, cuDoubleComplex y);





