#include <your_pm.hh>

using namespace std;
using namespace Array;
using namespace fftwpp;

typedef array2<Complex> cmplxArr2D; // Complex is defined in fftw++.h
typedef array1<Complex> cmplxArr1D; // Complex is defined in fftw++.h



/* 3D Vector Propagation Method */

void YOUR_vectorial_3D( cpx* EX, cpx* EY, cpx* EZ,                                   // electric vector field
			double* F,                                                   // energy flux
			const cpx* N,                                                // system refractive index distribution
			const cpx* KX, const double* X,                              // X-axis spatial frequencies KX and space X
			const cpx* KY, const double* Y,                              // Y-axis spatial frequencies KX and space X
			const double k0,                                             // wave number
			const double dz, const double dy, const double dx,           // spatial sampling
			const int nz, const int ny, const int nx,                    // number of nodes per axis
			const bool do_evanescent_modes,                              // configuration
			const bool is_debug ){

  if(is_debug)
  cout << " your_pm.cc: function YOUR_vectorial_3D()";

  /* complex number example
     cpx c;                              // c.real(), c.imag();
     cps ione(0,1), one(1,0), zero(0,0); 
  */

  /* FFT example
     cpx *SX = static_cast<cpx*>(malloc((nx*ny)*sizeof(cpx)));
     fft2d *forwardX = new fft2d(nx,ny,-1,NULL,SX);
     forwardX->fft(EX+(iz*(nx*ny)),SX);
     free(SX);
  */

}

/* 2D Vector Propagation Method */

void YOUR_vectorial_2D( cpx* EX, cpx* EY, cpx* EZ, double* F, 
			const cpx* N, const cpx* KX, const double* X, 
			const double k0, const double dz, const double dx, 
			const int nz, const int nx,
			const bool do_evanescent_modes,
			const bool is_debug ){

  if(is_debug)
    cout << "your_pm.cc: function YOUR_vectorial_2D()";

}

/* 3D Scalar Propagation Method */

void YOUR_scalar_3D( cpx* E, double* F, const cpx* N, 
		     const cpx* KX, const double* X, const cpx* KY, const double* Y, 
		     const double k0, const double dz, const double dy, const double dx, 
		     const int nz, const int ny, const int nx,
		     const bool do_evanescent_modes, const bool is_TE,
		     const bool is_debug ){

  if(is_debug)
  cout << "your_pm.cc: function YOUR_scalar_3D()";

}

/* 2D Scalar Propagation Method */

void YOUR_scalar_2D( cpx* E, double* F, const cpx* N, 
		     const cpx* KX, const double* X, 
		     const double k0, const double dz, const double dx, 
		     const int nz, const int nx,
		     const bool do_evanescent_modes, const bool is_TE,
		     const bool is_debug ){

  /* check the Makefile for DEBUG preprocessor variable */
  
#ifdef DEBUG
  if(is_debug)
    cout << "your_pm.cc: function YOUR_scalar_2D( )";
#endif

  /* example:
  const cpx one(1,0), ione(0,1);

  cpx *S = static_cast<cpx*>(malloc((nx)*sizeof(cpx)));
  fft1d *forward = new fft1d(nx,-1,NULL,S);
  fft1d *backward = new fft1d(nx,1,NULL,S);
  */

#ifdef TIMER
  ChTimer timer;
#endif

  for(int k=0; k<(nz-1); k++) { // loop something ...

    /* example: 
       forward->fft(E+base,S);
    */

    /* use pre-processor conditional code for debugging, timing analysis */

#ifdef DEBUG
    /* put your debugging code here */
#endif

#ifdef TIMER
    if(0==k) timer.start();
#endif

    for(int j=0; j<nx-1; j++) { // loop something else ...

#ifdef DEBUG
      /* or here */
#endif
     
    } // end j
   
#ifdef TIMER
    timer.stop();
    cout << 1e3 * timer.getTime();
#endif

    /* example:
    backward->fftNormalized(S,E+nbase);
    */

  } // end k
    
}
